# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2013 Hewlett-Packard Development Company, L.P.
# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""SQLAlchemy storage backend."""

import hashlib
import sys

from sqlalchemy.orm import exc

from ripcord.common import exception
from ripcord.db.sqlalchemy import models
from ripcord.openstack.common.db import exception as db_exc
from ripcord.openstack.common.db.sqlalchemy import session as db_session
from ripcord.openstack.common import log as logging
from ripcord.openstack.common import uuidutils

LOG = logging.getLogger(__name__)

get_session = db_session.get_session


def get_backend():
    """The backend is this module itself."""
    return sys.modules[__name__]


def model_query(model, *args, **kwargs):
    """Query helper for simpler session usage.

    :param session: if present, the session to use
    """

    session = kwargs.get('session') or get_session()
    query = session.query(model, *args)
    return query


def create_subscriber(
        username, domain, password, user, project, disabled=False, email='',
        rpid=''):
    """Create a new subscriber."""
    values = {
        'disabled': disabled,
        'domain': domain,
        'email_address': email,
        'password': password,
        'project_id': project,
        'rpid': rpid,
        'user_id': user,
        'username': username,
    }

    values['ha1'] = hashlib.md5(
        '%s:%s:%s' % (
            values['username'], values['domain'],
            values['password'])).hexdigest()
    values['ha1b'] = hashlib.md5(
        '%s@%s:%s:%s' % (
            values['username'], values['domain'], values['domain'],
            values['password'])).hexdigest()
    values['uuid'] = uuidutils.generate_uuid()

    try:
        res = _create_model(model=models.Subscriber(), values=values)
    except db_exc.DBDuplicateEntry:
        raise exception.SubscriberAlreadyExists(
            username=values['username'], domain=values['domain'])

    return res


def delete_subscriber(uuid):
    """Delete a subscriber."""
    res = _delete_model(model=models.Subscriber, uuid=uuid)

    if res != 1:
        raise exception.SubscriberNotFound(uuid=uuid)


def get_subscriber(uuid):
    """Retrieve information about the given subscriber."""
    try:
        res = _get_model(model=models.Subscriber, uuid=uuid)
    except exc.NoResultFound:
        raise exception.SubscriberNotFound(uuid=uuid)

    return res


def list_subscribers():
    """Retrieve a list of subscribers."""
    res = _list_model(model=models.Subscriber)

    return res


def update_subscriber(
        uuid, disabled=None, domain=None, email=None, password=None,
        project=None, rpid=None, user=None, username=None):
    """Update an existing subscriber."""
    res = get_subscriber(uuid=uuid)

    if disabled is not None:
        res['disabled'] = disabled
    if domain is not None:
        res['domain'] = domain
    if email is not None:
        res['email_address'] = email
    if password is not None:
        res['password'] = password
    if project is not None:
        res['project_id'] = project
    if rpid is not None:
        res['rpid'] = rpid
    if user is not None:
        res['user_id'] = user
    if username is not None:
        res['username'] = username

    res['ha1'] = hashlib.md5(
        '%s:%s:%s' % (
            res['username'], res['domain'],
            res['password'])).hexdigest()
    res['ha1b'] = hashlib.md5(
        '%s@%s:%s:%s' % (
            res['username'], res['domain'], res['domain'],
            res['password'])).hexdigest()

    res.save()

    return res


def _create_model(model, values):
    """Create a new model."""
    model.update(values)
    model.save()

    return model


def _delete_model(model, **kwargs):
    session = get_session()
    with session.begin():
        query = model_query(
            model, session=session
        ).filter_by(**kwargs)

        count = query.delete()

        return count


def _get_model(model, **kwargs):
    """Retrieve information about the given model."""
    query = model_query(model).filter_by(**kwargs)
    res = query.one()

    return res


def _list_model(model, **kwargs):
    """Retrieve a list of the given model."""
    query = model_query(model).filter_by(**kwargs)

    return query.all()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
  CLI interface for ripcord management.
"""

import logging

from oslo.config import cfg

from ripcord.common import config
from ripcord.db import migration as db_migration
from ripcord.openstack.common import log

CONF = cfg.CONF
LOG = log.getLogger(__name__)


def do_db_version():
    """Print database's current migration level."""
    print(db_migration.db_version())


def do_db_sync():
    """Place a database under migration control and upgrade,
    creating first if necessary.
    """
    db_migration.db_sync(CONF.command.version)


def add_command_parsers(subparsers):
    parser = subparsers.add_parser('db-version')
    parser.set_defaults(func=do_db_version)

    parser = subparsers.add_parser('db-sync')
    parser.set_defaults(func=do_db_sync)
    parser.add_argument('version', nargs='?')
    parser.add_argument('current_version', nargs='?')

    parser.add_argument(
        '-g', '--granularity', default='days',
        choices=['days', 'hours', 'minutes', 'seconds'],
        help='Granularity to use for age argument, defaults to days.')

command_opt = cfg.SubCommandOpt('command',
                                title='Commands',
                                help='Available commands',
                                handler=add_command_parsers)


def main():
    CONF.register_cli_opt(command_opt)
    config.parse_args()
    log.setup('ripcord')
    CONF.log_opt_values(LOG, logging.INFO)

    try:
        CONF.command.func()
    except Exception as e:
        LOG.exception(e)



# vim: tabstop=4 shiftwidth=4 softtabstop=4
# -*- encoding: utf-8 -*-

# Copyright © 2012 New Dream Network, LLC (DreamHost)
# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pecan

from oslo.config import cfg

from ripcord.api import acl
from ripcord.api import config
from ripcord.api import hooks
from ripcord.api import middleware

auth_opts = [
    cfg.StrOpt(
        'auth_strategy', default='keystone',
        help='The strategy to use for auth: noauth or keystone.'),
]

CONF = cfg.CONF
CONF.register_opts(auth_opts)


def get_pecan_config():
    # Set up the pecan configuration
    filename = config.__file__.replace('.pyc', '.py')
    return pecan.configuration.conf_from_file(filename)


def setup_app(pecan_config=None):
    app_hooks = [
        hooks.DBHook(),
    ]

    if not pecan_config:
        pecan_config = get_pecan_config()

    pecan.configuration.set_config(dict(pecan_config, overwrite=True))

    static_root = None
    if CONF.debug:
        static_root = pecan_config.app.static_root,

    app = pecan.make_app(
        pecan_config.app.root,
        static_root=static_root,
        template_path=pecan_config.app.template_path,
        debug=CONF.debug,
        force_canonical=getattr(pecan_config.app, 'force_canonical', True),
        hooks=app_hooks,
        wrap_app=middleware.ParsableErrorMiddleware,
    )

    if pecan_config.app.enable_acl:
        return acl.install(app, cfg.CONF)

    return app


class VersionSelectorApplication(object):
    def __init__(self):
        pc = get_pecan_config()
        pc.app.enable_acl = (CONF.auth_strategy == 'keystone')
        self.v1 = setup_app(pecan_config=pc)

    def __call__(self, environ, start_response):
        return self.v1(environ, start_response)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sqlalchemy import Boolean
from sqlalchemy import Column
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Integer
from sqlalchemy import schema
from sqlalchemy import String

from ripcord.openstack.common.db.sqlalchemy import models


class RipcordBase(models.TimestampMixin, models.ModelBase):

    metadata = None


Base = declarative_base(cls=RipcordBase)


class Subscriber(Base):
    __tablename__ = 'subscribers'
    __table_args__ = (
        schema.UniqueConstraint(
            'username', 'domain', name='uniq_subscriber0username0domain'),)

    id = Column(Integer, primary_key=True, autoincrement=True)
    disabled = Column(Boolean, default=False)
    domain = Column(String(64), nullable=False, default='')
    email_address = Column(String(64), nullable=False, default='')
    ha1 = Column(String(64), nullable=False, default='')
    ha1b = Column(String(64), nullable=False, default='')
    password = Column(String(25), nullable=False, default='')
    project_id = Column(String(255))
    rpid = Column(String(64))
    user_id = Column(String(255))
    username = Column(String(64), nullable=False, default='')
    uuid = Column(String(255), unique=True)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sqlalchemy import Boolean
from sqlalchemy import Column
from sqlalchemy import MetaData
from sqlalchemy import Table
from sqlalchemy import text
from sqlalchemy import UniqueConstraint

from ripcord.openstack.common import log as logging

LOG = logging.getLogger(__name__)

COLUMN_NAME = 'disabled'
TABLE_NAME = 'subscribers'


def upgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    t = Table(TABLE_NAME, meta, autoload=True)
    default = text('0') if migrate_engine.name == 'sqlite' else text('false')
    preserve_ephemeral_col = Column(
        COLUMN_NAME, Boolean, server_default=default)
    t.create_column(preserve_ephemeral_col)


def downgrade(migrate_engine):
    meta = MetaData()
    meta.bind = migrate_engine

    # NOTE(pabelanger): We need to setup our UniqueConstraint again, otherwise
    # autoload=True doesn't seem to pick it up.
    t = Table(
        TABLE_NAME, meta,
        UniqueConstraint(
            'username', 'domain',
            name='uniq_subscriber0username0domain'),
        autoload=True)

    # NOTE(pabelanger): SQLite doesn't have native BOOLEAN type, so it's
    # emulated by adding a CHECK constraint. We must explicitly omit that
    # constraint here so we don't receive 'no such column' error when dropping
    # the column.
    if migrate_engine.name == 'sqlite':
        t.constraints = set([
            c
            for c in t.constraints
            if not (hasattr(c, 'sqltext') and COLUMN_NAME in str(c.sqltext))
        ])

    t.drop_column(COLUMN_NAME)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from ripcord.openstack.common.db.sqlalchemy import migration as oslo_migration

INIT_VERSION = 0


def db_sync(version=None):
    path = os.path.join(
        os.path.abspath(os.path.dirname(__file__)), 'migrate_repo')
    return oslo_migration.db_sync(path, version, init_version=INIT_VERSION)


def db_version():
    path = os.path.join(
        os.path.abspath(os.path.dirname(__file__)), 'migrate_repo')
    return oslo_migration.db_version(path, INIT_VERSION)


def db_version_control(version=None):
    path = os.path.join(
        os.path.abspath(os.path.dirname(__file__)), 'migrate_repo')
    return oslo_migration.db_version_control(path, version)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ripcord.openstack.common import uuidutils
from ripcord.tests.api.v1 import base


class TestCase(base.FunctionalTest):

    def test_failure(self):
        res = self.put_json(
            '/subscribers/%s' % '0eda016a-b078-4bef-94ba-1ab10fe15a7d',
            status=404, expect_errors=True)

        self.assertEqual(res.status_int, 404)
        self.assertTrue(res.json['error_message'])

    def test_all_fields(self):
        json = {
            'disabled': True,
            'domain': 'example.org',
            'email_address': '',
            'ha1': '84ed3e3a76703c1044da21c8609334a2',
            'ha1b': '2dc0ac0e03670d8474db6b1e62df8fd1',
            'password': 'foobar',
            'project_id': '5fccabbb-9d65-417f-8b0b-a2fc77b501e6',
            'rpid': '',
            'updated_at': None,
            'user_id': '09f07543-6dad-441b-acbf-1c61b5f4015e',
            'username': 'alice',
        }
        params = {
            'disabled': json['disabled'],
            'domain': json['domain'],
            'email_address': json['email_address'],
            'password': json['password'],
            'rpid': json['rpid'],
            'username': json['username'],
        }
        headers = {
            'X-User-Id': json['user_id'],
            'X-Tenant-Id': json['project_id'],
        }

        tmp = self.post_json(
            '/subscribers', params=params, status=200, headers=headers)
        self.assertTrue(uuidutils.is_uuid_like(tmp.json['uuid']))

        json = {
            'disabled': False,
            'domain': 'example.net',
            'email_address': 'bob@example.net',
            'ha1': '9ea0e7b974be5095939ab2e6795f0159',
            'ha1b': '335bfe1e9cb562b4c76f285b69e9f9fa',
            'password': 'secret',
            'project_id': '09f07543-6dad-441b-acbf-1c61b5f4015e',
            'rpid': 'bob@example.net',
            'user_id': '5fccabbb-9d65-417f-8b0b-a2fc77b501e6',
            'username': 'bob',
        }
        params = {
            'disabled': json['disabled'],
            'domain': json['domain'],
            'email_address': json['email_address'],
            'password': json['password'],
            'rpid': json['rpid'],
            'username': json['username'],
        }
        headers = {
            'X-User-Id': json['user_id'],
            'X-Tenant-Id': json['project_id'],
        }
        res = self.put_json(
            '/subscribers/%s' % tmp.json['uuid'], params=params, status=200,
            headers=headers)

        for k, v in json.iteritems():
            self.assertEqual(res.json[k], v)

        self.assertTrue(res.json['created_at'])
        self.assertTrue(res.json['updated_at'])
        self.assertTrue(uuidutils.is_uuid_like(res.json['uuid']))

        # NOTE(pabelanger): We add 3 because of created_at, updated_at, and
        # uuid
        self.assertEqual(len(res.json), len(json) + 3)

    def test_no_fields(self):
        json = {
            'disabled': False,
            'domain': 'example.org',
            'email_address': '',
            'ha1': '84ed3e3a76703c1044da21c8609334a2',
            'ha1b': '2dc0ac0e03670d8474db6b1e62df8fd1',
            'password': 'foobar',
            'project_id': '5fccabbb-9d65-417f-8b0b-a2fc77b501e6',
            'rpid': '',
            'updated_at': None,
            'user_id': '09f07543-6dad-441b-acbf-1c61b5f4015e',
            'username': 'alice',
        }
        params = {
            'disabled': json['disabled'],
            'domain': json['domain'],
            'email_address': json['email_address'],
            'password': json['password'],
            'rpid': json['rpid'],
            'username': json['username'],
        }
        headers = {
            'X-User-Id': json['user_id'],
            'X-Tenant-Id': json['project_id'],
        }

        tmp = self.post_json(
            '/subscribers', params=params, status=200, headers=headers)
        self.assertTrue(uuidutils.is_uuid_like(tmp.json['uuid']))

        res = self.put_json(
            '/subscribers/%s' % tmp.json['uuid'])

        for k, v in json.iteritems():
            self.assertEqual(res.json[k], v)

        self.assertTrue(res.json['created_at'])
        self.assertTrue(uuidutils.is_uuid_like(res.json['uuid']))

        # NOTE(pabelanger): We add 3 because of created_at and uuid
        self.assertEqual(len(res.json), len(json) + 2)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import fixtures
import os
import shutil
import testtools

from oslo.config import cfg

from ripcord.common import paths
from ripcord.db import migration
from ripcord.openstack.common.db.sqlalchemy import session
from ripcord.openstack.common import log as logging
from ripcord.tests import conf_fixture


TEST_OPTS = [
    cfg.StrOpt(
        'sqlite_clean_db', default='clean.sqlite',
        help='Filename of clean sqlite db',
    ),
]

CONF = cfg.CONF
CONF.register_opts(TEST_OPTS)
CONF.import_opt(
    'connection', 'ripcord.openstack.common.db.sqlalchemy.session',
    group='database'
)
CONF.import_opt('sqlite_db', 'ripcord.openstack.common.db.sqlalchemy.session')

logging.setup('ripcord')

_DB_CACHE = None


class Database(fixtures.Fixture):

    def __init__(self, db_session, db_migrate, sql_connection,
                 sqlite_db, sqlite_clean_db):
        self.sql_connection = sql_connection
        self.sqlite_db = sqlite_db
        self.sqlite_clean_db = sqlite_clean_db

        self.engine = db_session.get_engine()
        self.engine.dispose()
        conn = self.engine.connect()
        if sql_connection == "sqlite://":
            if db_migrate.db_version() > db_migrate.db_initial_version():
                return
        else:
            testdb = paths.state_path_rel(sqlite_db)
            if os.path.exists(testdb):
                return
        db_migrate.db_sync()
        if sql_connection == "sqlite://":
            conn = self.engine.connect()
            self._db = "".join(line for line in conn.connection.iterdump())
            self.engine.dispose()
        else:
            cleandb = paths.state_path_rel(sqlite_clean_db)
            shutil.copyfile(testdb, cleandb)

    def setUp(self):
        super(Database, self).setUp()

        if self.sql_connection == "sqlite://":
            conn = self.engine.connect()
            conn.connection.executescript(self._db)
            self.addCleanup(self.engine.dispose)
        else:
            shutil.copyfile(paths.state_path_rel(self.sqlite_clean_db),
                            paths.state_path_rel(self.sqlite_db))


class TestCase(testtools.TestCase):
    """Test case base class for all unit tests."""

    def setUp(self):
        """Run before each method to initialize test environment."""
        super(TestCase, self).setUp()

        self.log_fixture = self.useFixture(fixtures.FakeLogger())
        self.useFixture(conf_fixture.ConfFixture(CONF))

        global _DB_CACHE
        if not _DB_CACHE:
            _DB_CACHE = Database(
                session, migration, sql_connection=CONF.database.connection,
                sqlite_db=CONF.sqlite_db, sqlite_clean_db=CONF.sqlite_clean_db
            )
        self.useFixture(_DB_CACHE)

    def path_get(self, project_file=None):
        """Get the absolute path to a file. Used for testing the API.

        :param project_file: File whose path to return. Default: None.
        :returns: path to the specified file, or path to project root.
        """
        root = os.path.abspath(
            os.path.join(os.path.dirname(__file__), '..', '..')
        )

        if project_file:
            return os.path.join(root, project_file)
        else:
            return root

    def config(self, **kw):
        """Override some configuration values.

        The keyword arguments are the names of configuration options to
        override and their values.

        If a group argument is supplied, the overrides are applied to
        the specified configuration option group.

        All overrides are automatically cleared at the end of the current
        test by the fixtures cleanup process.
        """
        group = kw.pop('group', None)
        for k, v in kw.iteritems():
            CONF.set_override(k, v, group)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class Schema(object):

    def __init__(self, name, properties={}):
        self.name = name
        self.properties = properties

    def raw(self):
        raw = {
            'name': self.name,
            'properties': self.properties,
            'additionalProperties': False,
        }
        return raw



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ripcord.db import migration
from ripcord import test


class TestCase(test.TestCase):
    def setUp(self):
        super(TestCase, self).setUp()

    def test_sync_and_version(self):
        migration.db_sync()
        v = migration.db_version()
        self.assertTrue(v > migration.db_initial_version())



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2013 Hewlett-Packard Development Company, L.P.
# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Ripcord Service API
"""

import logging

from oslo.config import cfg
from wsgiref import simple_server

from ripcord.api import app
from ripcord.common import config
from ripcord.openstack.common import log

CONF = cfg.CONF

LOG = log.getLogger(__name__)


def main():
    config.parse_args()
    log.setup('ripcord')
    host = CONF.bind_host
    port = CONF.bind_port
    wsgi = simple_server.make_server(
        host, port, app.VersionSelectorApplication()
    )

    LOG.info('Serving on http://%s:%s' % (host, port))
    LOG.info('Configuration:')
    CONF.log_opt_values(LOG, logging.INFO)

    try:
        wsgi.serve_forever()
    except KeyboardInterrupt:
        pass



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Ripcord base exception handling.
"""

from oslo.config import cfg

from ripcord.openstack.common import log as logging

LOG = logging.getLogger(__name__)

exc_log_opts = [
    cfg.BoolOpt('fatal_exception_format_errors',
                default=False,
                help='make exception message format errors fatal'),
]

CONF = cfg.CONF
CONF.register_opts(exc_log_opts)


class RipcordException(Exception):
    """Base Ripcord Exception

    To correctly use this class, inherit from it and define
    a 'message' property. That message will get printf'd
    with the keyword arguments provided to the constructor.

    """
    message = ("An unknown exception occurred.")
    code = 500
    headers = {}
    safe = False

    def __init__(self, message=None, **kwargs):
        self.kwargs = kwargs

        if 'code' not in self.kwargs:
            try:
                self.kwargs['code'] = self.code
            except AttributeError:
                pass

        if not message:
            try:
                message = self.message % kwargs

            except Exception as e:
                # kwargs doesn't match a variable in the message
                # log the issue and the kwargs
                LOG.exception('Exception in string format operation')
                for name, value in kwargs.iteritems():
                    LOG.error("%s: %s" % (name, value))

                if CONF.fatal_exception_format_errors:
                    raise e
                else:
                    # at least get the core message out if something happened
                    message = self.message

        super(RipcordException, self).__init__(message)

    def format_message(self):
        if self.__class__.__name__.endswith('_Remote'):
            return self.args[0]
        else:
            return unicode(self)


class Conflict(RipcordException):
    code = 409
    message = 'Conflict'


class NotFound(RipcordException):
    message = 'Resource could not be found'
    code = 404


class SubscriberAlreadyExists(Conflict):
    message = ('A subscriber with username %(username) and domain %(domains) '
               'already exists.')


class SubscriberNotFound(NotFound):
    message = 'Subscriber %(uuid)s could not be found'



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright 2012 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from oslo.config import cfg

path_opts = [
    cfg.StrOpt('pybasedir',
               default=os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                    '../')),
               help='Directory where the nova python module is installed'),
    cfg.StrOpt('bindir',
               default='$pybasedir/bin',
               help='Directory where nova binaries are installed'),
    cfg.StrOpt('state_path',
               default='$pybasedir',
               help="Top-level directory for maintaining nova's state"),
]

CONF = cfg.CONF
CONF.register_opts(path_opts)


def basedir_def(*args):
    """Return an uninterpolated path relative to $pybasedir."""
    return os.path.join('$pybasedir', *args)


def bindir_def(*args):
    """Return an uninterpolated path relative to $bindir."""
    return os.path.join('$bindir', *args)


def state_path_def(*args):
    """Return an uninterpolated path relative to $state_path."""
    return os.path.join('$state_path', *args)


def basedir_rel(*args):
    """Return a path relative to $pybasedir."""
    return os.path.join(CONF.pybasedir, *args)


def bindir_rel(*args):
    """Return a path relative to $bindir."""
    return os.path.join(CONF.bindir, *args)


def state_path_rel(*args):
    """Return a path relative to $state_path."""
    return os.path.join(CONF.state_path, *args)



# -*- encoding: utf-8 -*-
#
# Copyright © 2012 New Dream Network, LLC (DreamHost)
#
# Author: Doug Hellmann <doug.hellmann@dreamhost.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Access Control Lists (ACL's) control access the API server."""

from keystoneclient.middleware import auth_token
from oslo.config import cfg

#from ripcord.openstack.common import policy

_ENFORCER = None
OPT_GROUP_NAME = 'keystone_authtoken'


def register_opts(conf):
    """Register keystoneclient middleware options
    """
    conf.register_opts(auth_token.opts,
                       group=OPT_GROUP_NAME)
    auth_token.CONF = conf


register_opts(cfg.CONF)


def install(app, conf):
    """Install ACL check on application."""
    return auth_token.AuthProtocol(app,
                                   conf=dict(conf.get(OPT_GROUP_NAME)))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import fixtures

from oslo.config import cfg

from ripcord.common import config

CONF = cfg.CONF


class ConfFixture(fixtures.Fixture):
    """Fixture to manage global conf settings."""

    def __init__(self, conf):
        self.conf = conf

    def setUp(self):
        super(ConfFixture, self).setUp()

        self.conf.set_default('connection', "sqlite://", group='database')
        self.conf.set_default('sqlite_synchronous', False)
        self.conf.set_default('verbose', True)
        config.parse_args([], default_config_files=[])
        self.addCleanup(self.conf.reset)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo.config import cfg

from ripcord.openstack.common import log as logging

LOG = logging.getLogger(__name__)


API_SERVICE_OPTS = [
    cfg.StrOpt(
        'bind_host', default='0.0.0.0', help='The host IP to bind to'
    ),
    cfg.IntOpt('bind_port', default=9869, help='The port to bind to'),
]

CONF = cfg.CONF
CONF.register_opts(API_SERVICE_OPTS)

# Server Specific Configurations
server = {
    'port': CONF.bind_port,
    'host': CONF.bind_host,
}

# Pecan Application Configurations
app = {
    'root': 'ripcord.api.controllers.root.RootController',
    'modules': ['ripcord.api'],
    'static_root': '%(confdir)s/public',
    'template_path': '%(confdir)s/ripcord/api/templates',
    'enable_acl': True,
}



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo.config import cfg

from ripcord.openstack.common.db import api as db_api
from ripcord.openstack.common import log as logging

CONF = cfg.CONF

_BACKEND_MAPPING = {'sqlalchemy': 'ripcord.db.sqlalchemy.api'}

IMPL = db_api.DBAPI(backend_mapping=_BACKEND_MAPPING)
LOG = logging.getLogger(__name__)


def create_subscriber(
        username, domain, password, user, project, disabled=False, email='',
        rpid=''):
    return IMPL.create_subscriber(
        username=username, domain=domain, password=password, user=user,
        project=project, disabled=disabled, email=email, rpid=rpid)


def delete_subscriber(uuid):
    return IMPL.delete_subscriber(uuid=uuid)


def get_subscriber(uuid):
    return IMPL.get_subscriber(uuid=uuid)


def list_subscribers():
    return IMPL.list_subscribers()


def update_subscriber(
        uuid, disabled=None, domain=None, email=None, password=None,
        project=None, rpid=None, user=None, username=None):
    return IMPL.update_subscriber(
        uuid, disabled=disabled, domain=domain, email=email,
        password=password, project=project, rpid=rpid, user=user,
        username=username)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2013 Hewlett-Packard Development Company, L.P.
# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Base class for API tests."""

import pecan
import pecan.testing

from ripcord.openstack.common import log as logging
from ripcord.openstack.common import uuidutils
from ripcord.tests import base


LOG = logging.getLogger(__name__)


class FunctionalTest(base.TestCase):

    PATH_PREFIX = '/v1'

    def setUp(self):
        super(FunctionalTest, self).setUp()
        self.app = self._make_app()
        self.auth_headers = {
            'X-User-Id': uuidutils.generate_uuid(),
            'X-Tenant-Id': uuidutils.generate_uuid(),
        }

    def tearDown(self):
        super(FunctionalTest, self).tearDown()
        pecan.set_config({}, overwrite=True)

    def _make_app(self):
        root_dir = self.path_get()

        config = {
            'app': {
                'root': 'ripcord.api.controllers.root.RootController',
                'modules': ['ripcord.api'],
                'static_root': '%s/public' % root_dir,
                'template_path': '%s/api/templates' % root_dir,
                'enable_acl': False,
            },
        }

        return pecan.testing.load_test_app(config)

    def delete(self, path, expect_errors=False, headers=None,
               extra_environ=None, status=None):
        full_path = self.PATH_PREFIX + path
        LOG.debug('DELETE: %s' % (full_path))
        response = self.app.delete(str(full_path),
                                   headers=headers,
                                   status=status,
                                   extra_environ=extra_environ,
                                   expect_errors=expect_errors)
        LOG.debug('GOT: %s' % response)
        return response

    def get_json(self, path, expect_errors=False, headers=None,
                 extra_environ=None, q=[], **params):
        full_path = self.PATH_PREFIX + path
        query_params = {'q.field': [],
                        'q.value': [],
                        'q.op': [],
                        }
        for query in q:
            for name in ['field', 'op', 'value']:
                query_params['q.%s' % name].append(query.get(name, ''))
        all_params = {}
        all_params.update(params)
        if q:
            all_params.update(query_params)
        LOG.debug('GET: %s %r' % (full_path, all_params))
        response = self.app.get(full_path,
                                params=all_params,
                                headers=headers,
                                extra_environ=extra_environ,
                                expect_errors=expect_errors)
        if not expect_errors:
            response = response.json
        LOG.debug('GOT: %s' % response)
        return response

    def post_json(self, path, params={}, expect_errors=False, headers=None,
                  method="post", extra_environ=None, status=None):
        full_path = self.PATH_PREFIX + path
        LOG.debug('%s: %s %s' % (method.upper(), full_path, params))
        response = getattr(self.app, "%s_json" % method)(
            str(full_path),
            params=params,
            headers=headers,
            status=status,
            extra_environ=extra_environ,
            expect_errors=expect_errors
        )
        LOG.debug('GOT: %s' % response)
        return response

    def put_json(self, path, params={}, expect_errors=False, headers=None,
                 extra_environ=None, status=None):
        return self.post_json(path=path, params=params,
                              expect_errors=expect_errors,
                              headers=headers, extra_environ=extra_environ,
                              status=status, method="put")



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import wsme

from wsme import types


class APIBase(types.Base):

    created_at = datetime.datetime
    updated_at = datetime.datetime

    def as_dict(self):
        return dict((k, getattr(self, k))
                    for k in self.fields
                    if hasattr(self, k) and
                    getattr(self, k) != wsme.Unset)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime

from ripcord.common import exception
from ripcord.openstack.common import uuidutils
from ripcord.tests.db import base


class TestCase(base.FunctionalTest):

    def test_failure(self):
        self.assertRaises(
            exception.SubscriberNotFound,
            self.db_api.get_subscriber,
            '0eda016a-b078-4bef-94ba-1ab10fe15a7d')

    def test_success(self):
        row = {
            'disabled': False,
            'domain': 'example.org',
            'email_address': 'alice@example.org',
            'ha1': '84ed3e3a76703c1044da21c8609334a2',
            'ha1b': '2dc0ac0e03670d8474db6b1e62df8fd1',
            'id': 1,
            'password': 'foobar',
            'project_id': '793491dd5fa8477eb2d6a820193a183b',
            'rpid': 'alice@example.org',
            'updated_at': None,
            'user_id': '02d99a62af974b26b510c3564ba84644',
            'username': 'alice',
        }
        tmp = self.db_api.create_subscriber(
            username=row['username'], domain=row['domain'],
            password=row['password'], user=row['user_id'],
            project=row['project_id'], email=row['email_address'],
            rpid=row['rpid'])

        res = self.db_api.get_subscriber(
            uuid=tmp['uuid'])

        for k, v in row.iteritems():
            self.assertEqual(res[k], v)

        self.assertEqual(type(res['created_at']), datetime.datetime)
        self.assertTrue(uuidutils.is_uuid_like(res['uuid']))

        # NOTE(pabelanger): We add 3 because of created_at, uuid, and hidden
        # sqlalchemy object.
        self.assertEqual(len(res.__dict__), len(row) + 3)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 OpenStack LLC.
# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Routines for configuring Ripcord
"""

from oslo.config import cfg

from ripcord.common import paths
from ripcord.openstack.common.db.sqlalchemy import session as db_session
from ripcord import version

CONF = cfg.CONF
_DEFAULT_SQL_CONNECTION = 'sqlite:///' + paths.state_path_def('$sqlite_db')


def parse_args(args=None, usage=None, default_config_files=None):
    db_session.set_defaults(
        sql_connection=_DEFAULT_SQL_CONNECTION, sqlite_db='ripcord.sqlite'
    )
    CONF(
        args=args,
        project='ripcord',
        version=version.VERSION_INFO,
        usage=usage,
        default_config_files=default_config_files
    )



#!/usr/bin/env python

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys


sys.path.insert(0, os.path.abspath('../..'))

extensions = ['sphinx.ext.autodoc']
templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'
project = u'ripcord'
copyright = u'2013, Paul Belanger'
pygments_style = 'sphinx'
html_theme = 'default'
htmlhelp_basename = 'ripcordedoc'

latex_elements = {
}

latex_documents = [
    ('index', 'ripcord.tex', u'Ripcord Documentation',
     u'Paul Belanger', 'manual'),
]

man_pages = [
    ('man/ripcord-api', 'ripcord-api', u'Ripcord Documentation',
     [u'Paul Belanger'], 1),
    ('man/ripcord-manage', 'ripcord-manage', u'Ripcord Documentation',
     [u'Paul Belanger'], 1),
]

texinfo_documents = [
    ('index', 'ripcord', u'Ripcord Documentation',
     u'Paul Belanger', 'ripcord', 'One line description of project.',
     'Miscellaneous'),
]



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 PolyBeacon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pecan
import wsme

from pecan import rest
from wsme import types as wtypes
from wsmeext import pecan as wsme_pecan

from ripcord.api.controllers.v1 import base
from ripcord.common import exception
from ripcord.db.sqlalchemy import models
from ripcord.openstack.common import log as logging

LOG = logging.getLogger(__name__)


class Subscriber(base.APIBase):
    """API representation of an subscriber."""

    disabled = bool
    domain = wtypes.text
    email_address = wtypes.text
    ha1 = wtypes.text
    ha1b = wtypes.text
    password = wtypes.text
    project_id = wtypes.text
    rpid = wtypes.text
    user_id = wtypes.text
    username = wtypes.text
    uuid = wtypes.text

    def __init__(self, **kwargs):
        self.fields = vars(models.Subscriber)
        for k in self.fields:
            setattr(self, k, kwargs.get(k))


class SubscribersController(rest.RestController):
    """REST Controller for Subscriber."""

    @wsme_pecan.wsexpose(None, wtypes.text, status_code=204)
    def delete(self, uuid):
        """Delete an subscriber."""
        try:
            pecan.request.db_api.delete_subscriber(uuid=uuid)
        except exception.SubscriberNotFound as e:
            raise wsme.exc.ClientSideError(e.message, status_code=e.code)

    @wsme_pecan.wsexpose([Subscriber])
    def get_all(self):
        """Retrieve a list of subscribers."""
        res = pecan.request.db_api.list_subscribers()

        return res

    @wsme_pecan.wsexpose(Subscriber, unicode)
    def get_one(self, uuid):
        """Retrieve information about the given subscriber."""
        try:
            result = pecan.request.db_api.get_subscriber(uuid)
        except exception.SubscriberNotFound as e:
            raise wsme.exc.ClientSideError(e.message, status_code=e.code)

        return result

    @wsme.validate(Subscriber)
    @wsme_pecan.wsexpose(Subscriber, body=Subscriber)
    def post(self, body):
        """Create a new subscriber."""
        try:
            user_id = pecan.request.headers.get('X-User-Id')
            project_id = pecan.request.headers.get('X-Tenant-Id')

            d = body.as_dict()

            res = pecan.request.db_api.create_subscriber(
                username=d['username'], domain=d['domain'],
                password=d['password'], user=user_id,
                project=project_id, disabled=d['disabled'],
                email=d['email_address'], rpid=d['rpid'])
        except exception.SubscriberAlreadyExists as e:
            raise wsme.exc.ClientSideError(e.message, status_code=e.code)

        return res

    @wsme.validate(Subscriber)
    @wsme_pecan.wsexpose(Subscriber, wtypes.text, body=Subscriber)
    def put(self, uuid, body):
        """Update an existing subscriber."""
        try:
            user_id = pecan.request.headers.get('X-User-Id')
            project_id = pecan.request.headers.get('X-Tenant-Id')

            d = body.as_dict()

            res = pecan.request.db_api.update_subscriber(
                uuid=uuid, username=d['username'], domain=d['domain'],
                password=d['password'], user=user_id, project=project_id,
                disabled=d['disabled'], email=d['email_address'],
                rpid=d['rpid'])
        except exception.SubscriberNotFound as e:
            raise wsme.exc.ClientSideError(e.message, status_code=e.code)

        return res
